@@ -1,8 +1,8 @@
 /*
- * $Id: main.c,v 1.124.2.1 2002/07/02 21:38:02 tal197 Exp $
+ * $Id: main.c,v 1.160 2003/01/19 19:29:04 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
- * Copyright (C) 2002, the ROX-Filer team.
+ * Copyright (C) 2003, the ROX-Filer team.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -68,13 +68,12 @@
 #include "run.h"
 #include "toolbar.h"
 #include "bind.h"
-#include "icon.h"
 #include "panel.h"
 #include "session.h"
 #include "minibuffer.h"
 
 int number_of_windows = 0;	/* Quit when this reaches 0 again... */
-static int to_wakeup_pipe = -1;	/* Write here to get noticed */
+int to_wakeup_pipe = -1;	/* Write here to get noticed */
 
 /* Information about the ROX-Filer process */
 uid_t euid;
@@ -83,15 +82,15 @@ int ngroups;			/* Number of supplemental groups */
 gid_t *supplemental_groups = NULL;
 
 /* Message to display at the top of each filer window */
-guchar *show_user_message = NULL;
+const gchar *show_user_message = NULL;
 
 int home_dir_len;
-char *home_dir, *app_dir;
+const char *home_dir, *app_dir;
 
 GtkTooltips *tooltips = NULL;
 
 #define COPYING								\
-	     N_("Copyright (C) 2002 Thomas Leonard.\n"			\
+	     N_("Copyright (C) 2003 Thomas Leonard.\n"			\
 		"ROX-Filer comes with ABSOLUTELY NO WARRANTY,\n"	\
 		"to the extent permitted by law.\n"			\
 		"You may redistribute copies of ROX-Filer\n"		\
@@ -119,8 +118,7 @@ GtkTooltips *tooltips = NULL;
        "  -h, --help		display this help and exit\n"		\
        "  -l, --left=PANEL	open PAN as a left-edge panel\n"	\
        "  -m, --mime-type=FILE	print MIME type of FILE and exit\n" \
-       "  -n, --new		start a new filer, even if already running\n"  \
-       "  -o, --override	override window manager control of panels\n" \
+       "  -n, --new		start new copy; for debugging the filer\n"  \
        "  -p, --pinboard=PIN	use pinboard PIN as the pinboard\n"	\
        "  -r, --right=PANEL	open PAN as a right-edge panel\n"	\
        "  -R, --RPC		invoke method call read from stdin\n"	\
@@ -129,9 +127,8 @@ GtkTooltips *tooltips = NULL;
        "  -u, --user		show user name in each window \n"	\
        "  -v, --version		display the version information and exit\n"   \
        "  -x, --examine=FILE	FILE has changed - re-examine it\n"	\
-       "\nThe latest version can be found at:\n"			\
-       "\thttp://rox.sourceforge.net\n"					\
-       "\nReport bugs to <tal197@users.sourceforge.net>.\n")
+       "\nReport bugs to <tal197@users.sourceforge.net>.\n"		\
+       "Home page (including updated versions): http://rox.sourceforge.net/\n")
 
 #define SHORT_OPS "c:d:t:b:l:r:op:s:hvnux:m:D:R"
 
@@ -160,7 +157,7 @@ static struct option long_opts[] =
 #endif
 
 /* Take control of panels away from WM? */
-gboolean override_redirect = FALSE;
+Option o_override_redirect;
 
 /* Always start a new filer, even if one seems to be already running */
 gboolean new_copy = FALSE;
@@ -169,12 +166,14 @@ gboolean new_copy = FALSE;
 static GHashTable *death_callbacks = NULL;
 static gboolean child_died_flag = FALSE;
 
+Option o_dnd_no_hostnames;
+
 /* Static prototypes */
 static void show_features(void);
 static void soap_add(xmlNodePtr body,
 			   xmlChar *function,
-			   xmlChar *arg1_name, xmlChar *arg1_value,
-			   xmlChar *arg2_name, xmlChar *arg2_value);
+			   const xmlChar *arg1_name, const xmlChar *arg1_value,
+			   const xmlChar *arg2_name, const xmlChar *arg2_value);
 static void child_died(int signum);
 static void child_died_callback(void);
 static void wake_up_cb(gpointer data, gint source, GdkInputCondition condition);
@@ -196,7 +195,7 @@ int main(int argc, char **argv)
 	int		 wakeup_pipe[2];
 	int		 i;
 	struct sigaction act;
-	guchar		*tmp, *dir, *slash;
+	guchar		*tmp, *dir;
 	gchar *client_id = NULL;
 	gboolean	show_user = FALSE;
 	xmlDocPtr	rpc, soap_rpc = NULL, reply;
@@ -233,10 +232,6 @@ int main(int argc, char **argv)
 	 */
 	death_callbacks = g_hash_table_new(NULL, NULL);
 
-#ifdef HAVE_LIBVFS
-	mc_vfs_init();
-#endif
-
 	/* Find out some information about ourself */
 	euid = geteuid();
 	egid = getegid();
@@ -255,12 +250,14 @@ int main(int argc, char **argv)
 		 * compiled OK. Do this test before gtk_init so that
 		 * we don't need an X server to install.
 		 */
-		fprintf(stderr, "ROX-Filer %s\n", VERSION);
-		fprintf(stderr, _(COPYING));
+		g_print("ROX-Filer %s\n", VERSION);
+		g_print(_(COPYING));
 		show_features();
 		return EXIT_SUCCESS;
 	}
 
+	option_add_int(&o_override_redirect, "override_redirect", FALSE);
+
 	/* The idea here is to convert the command-line arguments
 	 * into a SOAP RPC.
 	 * We attempt to invoke the call on an already-running copy of
@@ -271,8 +268,22 @@ int main(int argc, char **argv)
 	/* Note: must do this before checking our options,
 	 * otherwise we report an error for Gtk's options.
 	 */
-	add_default_styles();
 	gtk_init(&argc, &argv);
+	/* Set a default style for the collection widget */
+	gtk_rc_parse_string("style \"rox-default-collection-style\" {\n"
+		"  bg[NORMAL] = \"#f3f3f3\"\n"
+		"  fg[NORMAL] = \"#000000\"\n"
+		"  bg[INSENSITIVE] = \"#bfbfbf\"\n"
+		"  fg[INSENSITIVE] = \"#000000\"\n"
+		"}\n"
+		"style \"rox-default-pinboard-style\" {\n"
+		"  bg[NORMAL] = \"#666666\"\n"
+		"}\n"
+		"widget \"rox-pinboard\" style : gtk "
+		"\"rox-default-pinboard-style\"\n"
+
+		"class \"Collection\" style : gtk "
+		"\"rox-default-collection-style\"\n");
 
 	/* Process each option in turn */
 	while (1)
@@ -295,22 +306,28 @@ int main(int argc, char **argv)
 				new_copy = TRUE;
 				break;
 			case 'o':
-				override_redirect = TRUE;
+				info_message(_("The -o argument is no longer "
+					"used. You can turn on override "
+					"redirect from the Options box "
+					"instead."));
 				break;
 			case 'v':
-				fprintf(stderr, "ROX-Filer %s\n", VERSION);
-				fprintf(stderr, _(COPYING));
+				g_print("ROX-Filer %s\n", VERSION);
+				g_print("%s", _(COPYING));
 				show_features();
 				return EXIT_SUCCESS;
 			case 'h':
-				fprintf(stderr, _(HELP));
-				fprintf(stderr, _(SHORT_ONLY_WARNING));
+				g_print("%s", _(HELP));
+				g_print("%s", _(SHORT_ONLY_WARNING));
 				return EXIT_SUCCESS;
 			case 'D':
 			case 'd':
 		        case 'x':
 				/* Argument is a path */
-				tmp = pathdup(VALUE);
+				if (c == 'd' && VALUE[0] == '/')
+					tmp = g_strdup(VALUE);
+				else
+					tmp = pathdup(VALUE);
 				soap_add(body,
 					c == 'D' ? "CloseDir" :
 					c == 'd' ? "OpenDir" :
@@ -320,23 +337,16 @@ int main(int argc, char **argv)
 				g_free(tmp);
 				break;
 			case 's':
-				tmp = g_strdup(VALUE);
-				slash = strrchr(tmp, '/');
-				if (slash)
-				{
-					*slash = '\0';
-					slash++;
-					dir = pathdup(tmp);
-				}
+				tmp = g_path_get_dirname(VALUE);
+				
+				if (tmp[0] == '/')
+					dir = NULL;
 				else
-				{
-					slash = tmp;
-					dir = pathdup(".");
-				}
+					dir = pathdup(tmp);
 
 				soap_add(body, "Show",
-					"Directory", dir,
-					"Leafname", slash);
+					"Directory", dir ? dir : tmp,
+					"Leafname", g_basename(VALUE));
 				g_free(tmp);
 				g_free(dir);
 				break;
@@ -360,14 +370,14 @@ int main(int argc, char **argv)
 				show_user = TRUE;
 				break;
 		        case 'm':
-				{
-					MIME_type *type;
-					type_init();
-					type = type_get_type(VALUE);
-					printf("%s/%s\n", type->media_type,
-							  type->subtype);
-				}
+			{
+				MIME_type *type;
+				type_init();
+				type = type_get_type(VALUE);
+				printf("%s/%s\n", type->media_type,
+						type->subtype);
 				return EXIT_SUCCESS;
+			}
 			case 'c':
 				client_id = g_strdup(VALUE);
 				break;
@@ -385,8 +395,8 @@ int main(int argc, char **argv)
 	tooltips = gtk_tooltips_new();
 
 	if (euid == 0 || show_user)
-		show_user_message = g_strdup_printf( _("Running as user '%s'"), 
-				user_name(euid));
+		show_user_message = g_strdup_printf(_("Running as user '%s'"), 
+						    user_name(euid));
 	
 	/* Add each remaining (non-option) argument to the list of files
 	 * to run.
@@ -421,7 +431,7 @@ int main(int argc, char **argv)
 		g_free(dir);
 	}
 
-	option_add_int("dnd_no_hostnames", 1, NULL);
+	option_add_int(&o_dnd_no_hostnames, "dnd_no_hostnames", 1);
 
 	/* Try to send the request to an already-running copy of the filer */
 	gui_support_init();
@@ -457,9 +467,9 @@ int main(int argc, char **argv)
 			close(fd);
 		}
 	}
-	
+
 	/* Initialize the rest of the filer... */
-	
+
 	pixmaps_init();
 
 	dnd_init();
@@ -475,7 +485,6 @@ int main(int argc, char **argv)
 	type_init();
 	action_init();
 
-	icon_init();
 	pinboard_init();
 	panel_init();
 
@@ -489,7 +498,8 @@ int main(int argc, char **argv)
 	pipe(wakeup_pipe);
 	close_on_exec(wakeup_pipe[0], TRUE);
 	close_on_exec(wakeup_pipe[1], TRUE);
-	gdk_input_add(wakeup_pipe[0], GDK_INPUT_READ, wake_up_cb, NULL);
+	gtk_input_add_full(wakeup_pipe[0], GDK_INPUT_READ, wake_up_cb,
+			NULL, NULL, NULL);
 	to_wakeup_pipe = wakeup_pipe[1];
 
 	/* If the pipe is full then we're going to get woken up anyway... */
@@ -510,7 +520,7 @@ int main(int argc, char **argv)
 	/* Set up session managament if available */
 	session_init(client_id);
 	g_free(client_id);
-		
+
 	/* Finally, execute the request */
 	reply = run_soap(rpc);
 	xmlFreeDoc(rpc);
@@ -545,64 +555,54 @@ void on_child_death(gint child, CallbackFn callback, gpointer data)
 	g_hash_table_insert(death_callbacks, GINT_TO_POINTER(child), cb);
 }
 
+void one_less_window(void)
+{
+	if (--number_of_windows < 1)
+		gtk_main_quit();
+}
+
 /****************************************************************
  *			INTERNAL FUNCTIONS			*
  ****************************************************************/
 
 static void show_features(void)
 {
-	g_printerr("\n-- %s --\n\n", _("features set at compile time"));
-	g_printerr("%s... %s\n", _("Large File Support"),
+	g_print("\n");
+	g_print(_("Compiled with GTK version %s\n"), GTK_VERSION);
+	g_print(_("Running with GTK version %d.%d.%d\n"),
+				gtk_major_version,
+				gtk_minor_version,
+				gtk_micro_version);
+	g_print("\n-- %s --\n\n", _("features set at compile time"));
+	g_print("%s... %s\n", _("Large File Support"),
 #ifdef LARGE_FILE_SUPPORT
 		_("Yes")
 #else
 		_("No")
 #endif
 		);
-	g_printerr("%s... %s\n", _("Old VFS support"),
-#ifdef HAVE_LIBVFS
-		_("Yes")
-#else
-# ifdef LARGE_FILE_SUPPORT
-		_("No (incompatible with Large File Support)")
-# else
-		_("No (couldn't find a valid libvfs)")
-# endif
-#endif
-		);
-	g_printerr("%s... %s\n", _("Gtk+-2.0 support"),
-#ifdef GTK2
-		_("Yes")
-#else
-		_("No (using Gtk+-1.2 instead)")
-#endif
-		);
-	g_printerr("%s... %s\n", _("Save thumbnails"),
-#ifdef GTK2
-		_("Yes (using Gtk+-2.0)")
-#elif defined(HAVE_LIBPNG)
-		_("Yes (using libpng)")
-#else
-		_("No (needs libpng or Gtk+-2.0)")
-#endif
-		);
-	g_printerr("%s... %s\n", _("Character set translations"),
-#ifdef GTK2
-			_("Yes (using Gtk+-2.0)")
-#else
-# ifdef HAVE_ICONV_H
+#if 0
+	g_print("%s... %s\n", _("GNOME-VFS library"),
+# ifdef WITH_GNOMEVFS
 			_("Yes")
 # else
-			_("No (needs libiconv or Gtk+-2.0)")
+			_("No (gnome-vfs-config not found)")
 # endif
-#endif
 		  );
+#endif
+	g_print("%s... %s\n", _("Dnotify support"),
+#ifdef USE_DNOTIFY
+		_("Yes")
+#else
+		_("No")
+#endif
+	       );
 }
 
 static void soap_add(xmlNodePtr body,
 			   xmlChar *function,
-			   xmlChar *arg1_name, xmlChar *arg1_value,
-			   xmlChar *arg2_name, xmlChar *arg2_value)
+			   const xmlChar *arg1_name, const xmlChar *arg1_value,
+			   const xmlChar *arg2_name, const xmlChar *arg2_value)
 {
 	xmlNodePtr node;
 	xmlNs *rox;
@@ -669,4 +669,8 @@ static void wake_up_cb(gpointer data, gint source, GdkInputCondition condition)
 	
 	if (child_died_flag)
 		child_died_callback();
+#ifdef USE_DNOTIFY
+	if (dnotify_wakeup)
+		dnotify_wakeup();
+#endif
 }
