@@ -1,6 +1,4 @@
 /*
- * $Id: mount.h,v 1.8 2000/12/17 18:49:23 tal197 Exp $
- *
  * ROX-Filer, filer for the ROX desktop project
  * By Thomas Leonard, <tal197@users.sourceforge.net>.
  */
@@ -24,8 +22,11 @@ struct _MountPoint
 };
 
 /* Prototypes */
-void mount_init();
+void mount_init(void);
 void mount_update(gboolean force);
-gboolean mount_is_mounted(guchar *path);
+void mount_user_mount(const char *path);
+gboolean mount_is_user_mounted(const gchar *path);
+gboolean mount_is_mounted(const guchar *path, struct stat *info,
+					      struct stat *parent);
 
 #endif /* _MOUNT_H */
