@@ -1,8 +1,8 @@
 /*
- * $Id: toolbar.c,v 1.27 2002/01/02 15:36:04 tal197 Exp $
+ * $Id: toolbar.c,v 1.71 2003/04/15 20:09:26 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
- * Copyright (C) 2002, the ROX-Filer team.
+ * Copyright (C) 2003, the ROX-Filer team.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -27,7 +27,6 @@
 
 #include "global.h"
 
-#include "collection.h"
 #include "toolbar.h"
 #include "options.h"
 #include "support.h"
@@ -41,24 +40,24 @@
 #include "type.h"
 #include "dir.h"
 #include "diritem.h"
+#include "view_iface.h"
+#include "bookmarks.h"
 
 typedef struct _Tool Tool;
 
-typedef enum {DROP_NONE, DROP_TO_PARENT, DROP_TO_HOME} DropDest;
+typedef enum {DROP_NONE, DROP_TO_PARENT, DROP_TO_HOME, DROP_BOOKMARK} DropDest;
 
 struct _Tool {
-	guchar		*label;
-	guchar		*name;
-	guchar		*tip;		/* Tooltip */
+	const gchar	*label;
+	const gchar	*name;
+	const gchar	*tip;		/* Tooltip */
 	void		(*clicked)(GtkWidget *w, FilerWindow *filer_window);
 	DropDest	drop_action;
 	gboolean	enabled;
-	MaskedPixmap	*icon;
-	GtkWidget	**menu;		/* Right-click menu widget addr */
+	gboolean	menu;		/* Activate on button-press */
 };
 
-ToolbarType o_toolbar = TOOLBAR_NORMAL;
-gint o_toolbar_info = TRUE;
+Option o_toolbar, o_toolbar_info, o_toolbar_disable;
 
 static GtkTooltips *tooltips = NULL;
 
@@ -66,22 +65,26 @@ static GtkTooltips *tooltips = NULL;
  * rather than reusing the existing one.
  */
 #define NEW_WIN_BUTTON(button_event)	\
-  (o_new_window_on_1 ? ((GdkEventButton *) button_event)->button == 1	\
-		     : ((GdkEventButton *) button_event)->button != 1)
+  (o_new_button_1.int_value		\
+   	? ((GdkEventButton *) button_event)->button == 1	\
+	: ((GdkEventButton *) button_event)->button != 1)
 
 /* Static prototypes */
 static void toolbar_close_clicked(GtkWidget *widget, FilerWindow *filer_window);
 static void toolbar_up_clicked(GtkWidget *widget, FilerWindow *filer_window);
 static void toolbar_home_clicked(GtkWidget *widget, FilerWindow *filer_window);
+static void toolbar_bookmarks_clicked(GtkWidget *widget,
+				      FilerWindow *filer_window);
 static void toolbar_help_clicked(GtkWidget *widget, FilerWindow *filer_window);
 static void toolbar_refresh_clicked(GtkWidget *widget,
 				    FilerWindow *filer_window);
 static void toolbar_size_clicked(GtkWidget *widget, FilerWindow *filer_window);
+static void toolbar_autosize_clicked(GtkWidget *widget, FilerWindow *filer_window);
 static void toolbar_details_clicked(GtkWidget *widget,
 				    FilerWindow *filer_window);
 static void toolbar_hidden_clicked(GtkWidget *widget,
 				   FilerWindow *filer_window);
-static GtkWidget *add_button(GtkWidget *box, Tool *tool,
+static GtkWidget *add_button(GtkWidget *bar, Tool *tool,
 				FilerWindow *filer_window);
 static GtkWidget *create_toolbar(FilerWindow *filer_window);
 static gboolean drag_motion(GtkWidget		*widget,
@@ -97,45 +100,51 @@ static void drag_leave(GtkWidget	*widget,
 static void handle_drops(FilerWindow *filer_window,
 			 GtkWidget *button,
 			 DropDest dest);
-static void coll_selection_changed(Collection *collection, guint time,
-					gpointer user_data);
-static void recreate_toolbar(FilerWindow *filer_window);
-static void toggle_shaded(GtkWidget *widget);
+static void toggle_selected(GtkToggleButton *widget, gpointer data);
 static void option_notify(void);
-static GList *build_tool_options(OptionUI *ui, xmlNode *node, guchar *label);
+static GList *build_tool_options(Option *option, xmlNode *node, guchar *label);
+static void tally_items(gpointer key, gpointer value, gpointer data);
 
 static Tool all_tools[] = {
-	{N_("Close"), "close", N_("Close filer window"),
+	{N_("Close"), GTK_STOCK_CLOSE, N_("Close filer window"),
 	 toolbar_close_clicked, DROP_NONE, FALSE,
-	 NULL, NULL},
+	 FALSE},
 	 
-	{N_("Up"), "up", N_("Change to parent directory"),
+	{N_("Up"), GTK_STOCK_GO_UP, N_("Change to parent directory"),
 	 toolbar_up_clicked, DROP_TO_PARENT, TRUE,
-	 NULL, NULL},
+	 FALSE},
 	 
-	{N_("Home"), "home", N_("Change to home directory"),
+	{N_("Home"), GTK_STOCK_HOME, N_("Change to home directory"),
 	 toolbar_home_clicked, DROP_TO_HOME, TRUE,
-	 NULL, NULL},
+	 FALSE},
 	
-	{N_("Scan"), "refresh", N_("Rescan directory contents"),
+	{N_("Bookmarks"), ROX_STOCK_BOOKMARKS, N_("Bookmarks menu"),
+	 toolbar_bookmarks_clicked, DROP_BOOKMARK, FALSE,
+	 TRUE},
+
+	{N_("Scan"), GTK_STOCK_REFRESH, N_("Rescan directory contents"),
 	 toolbar_refresh_clicked, DROP_NONE, TRUE,
-	 NULL, NULL},
+	 FALSE},
 	
-	{N_("Size"), "zoom", N_("Change icon size"),
+	{N_("Size"), GTK_STOCK_ZOOM_IN, N_("Change icon size"),
 	 toolbar_size_clicked, DROP_NONE, TRUE,
-	 NULL, NULL},
+	 FALSE},
 	
-	{N_("Details"), "details", N_("Show extra details"),
+	{N_("Size"), GTK_STOCK_ZOOM_FIT, N_("Automatic size mode"),
+	 toolbar_autosize_clicked, DROP_NONE, TRUE,
+	 FALSE},
+	
+	{N_("Details"), ROX_STOCK_SHOW_DETAILS, N_("Show extra details"),
 	 toolbar_details_clicked, DROP_NONE, TRUE,
-	 NULL, NULL},
+	 FALSE},
 	
-	{N_("Hidden"), "hidden", N_("Show/hide hidden files"),
+	{N_("Hidden"), ROX_STOCK_SHOW_HIDDEN, N_("Show/hide hidden files"),
 	 toolbar_hidden_clicked, DROP_NONE, TRUE,
-	 NULL, NULL},
+	 FALSE},
 	
-	{N_("Help"), "help", N_("Show ROX-Filer help"),
+	{N_("Help"), GTK_STOCK_HELP, N_("Show ROX-Filer help"),
 	 toolbar_help_clicked, DROP_NONE, TRUE,
-	 NULL, NULL},
+	 FALSE},
 };
 
 
@@ -145,92 +154,115 @@ static Tool all_tools[] = {
 
 void toolbar_init(void)
 {
-	int	i;
-	
-	option_add_int("toolbar_type", o_toolbar, NULL);
-	option_add_int("toolbar_show_info", o_toolbar_info, NULL);
-	option_add_string("toolbar_disable", "close", NULL);
+	option_add_int(&o_toolbar, "toolbar_type", TOOLBAR_NORMAL);
+	option_add_int(&o_toolbar_info, "toolbar_show_info", 1);
+	option_add_string(&o_toolbar_disable, "toolbar_disable",
+					GTK_STOCK_CLOSE);
 	option_add_notify(option_notify);
 	
 	tooltips = gtk_tooltips_new();
 
-	for (i = 0; i < sizeof(all_tools) / sizeof(*all_tools); i++)
-	{
-		Tool	*tool = &all_tools[i];
-
-		if (!tool->icon)
-		{
-			guchar	*path;
-
-			path = g_strconcat("pixmaps/",
-					tool->name, ".xpm", NULL);
-			tool->icon = load_pixmap(path);
-			g_free(path);
-		}
-	}
-
 	option_register_widget("tool-options", build_tool_options);
 }
 
-/* Returns a button which can be used to turn a tool on and off.
- * Button has 'tool_name' set to the tool's name.
- * NULL if tool number i is too big.
- */
-GtkWidget *toolbar_tool_option(int i)
-{
-	Tool		*tool = &all_tools[i];
-	GtkWidget	*button;
-	GtkWidget	*icon_widget, *vbox, *text;
-
-	g_return_val_if_fail(i >= 0, NULL);
-
-	if (i >= sizeof(all_tools) / sizeof(*all_tools))
-		return NULL;
-
-	button = gtk_button_new();
-	GTK_WIDGET_UNSET_FLAGS(button, GTK_CAN_FOCUS);
-	gtk_tooltips_set_tip(tooltips, button, _(tool->tip), NULL);
-
-	icon_widget = gtk_pixmap_new(tool->icon->pixmap,
-				     tool->icon->mask);
-
-	vbox = gtk_vbox_new(FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(vbox), icon_widget, TRUE, TRUE, 0);
-
-	text = gtk_label_new(_(tool->label));
-	gtk_box_pack_start(GTK_BOX(vbox), text, FALSE, TRUE, 0);
-
-	gtk_container_add(GTK_CONTAINER(button), vbox);
-
-	gtk_container_set_border_width(GTK_CONTAINER(button), 1);
-	gtk_misc_set_padding(GTK_MISC(icon_widget), 16, 1);
-
-	gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
-		GTK_SIGNAL_FUNC(toggle_shaded), GTK_OBJECT(vbox));
-
-	gtk_object_set_data(GTK_OBJECT(button), "tool_name", tool->name);
-
-	return button;
-}
-
-/* Create a new toolbar widget, suitable for adding to a filer window,
- * and return it.
- */
-GtkWidget *toolbar_new(FilerWindow *filer_window)
-{
-	g_return_val_if_fail(filer_window != NULL, NULL);
-	g_return_val_if_fail(o_toolbar != TOOLBAR_NONE, NULL);
-
-	return create_toolbar(filer_window);
-}
-
 void toolbar_update_info(FilerWindow *filer_window)
 {
-	if (o_toolbar != TOOLBAR_NONE && o_toolbar_info)
-		coll_selection_changed(filer_window->collection,
-				GDK_CURRENT_TIME, filer_window);
+	gchar		*label;
+	ViewIface	*view;
+	int		n_selected;
+
+	if (o_toolbar.int_value == TOOLBAR_NONE || !o_toolbar_info.int_value)
+		return;		/* Not showing info */
+
+	if (filer_window->target_cb)
+		return;
+
+	view = filer_window->view;
+
+	n_selected = view_count_selected(view);
+
+	if (n_selected == 0)
+	{
+		gchar *s = NULL;
+		int   n_items;
+
+		if (filer_window->scanning)
+		{
+			gtk_label_set_text(
+				GTK_LABEL(filer_window->toolbar_text), "");
+			return;
+		}
+
+		if (!filer_window->show_hidden)
+		{
+			GHashTable *hash = filer_window->directory->known_items;
+			int	   tally = 0;
+
+			g_hash_table_foreach(hash, tally_items, &tally);
+
+			if (tally)
+				s = g_strdup_printf(_(" (%u hidden)"), tally);
+		}
+
+		n_items = view_count_items(view);
+
+		if (n_items)
+			label = g_strdup_printf("%d %s%s",
+					n_items,
+					n_items != 1 ? _("items") : _("item"),
+					s ? s : "");
+		else /* (French plurals work differently for zero) */
+			label = g_strdup_printf(_("No items%s"),
+					s ? s : "");
+		g_free(s);
+	}
+	else
+	{
+		double	size = 0;
+		ViewIter iter;
+		DirItem *item;
+
+		view_get_iter(filer_window->view, &iter, VIEW_ITER_SELECTED);
+
+		while ((item = iter.next(&iter)))
+			if (item->base_type != TYPE_DIRECTORY)
+				size += (double) item->size;
+
+		label = g_strdup_printf(_("%u selected (%s)"),
+				n_selected, format_double_size(size));
+	}
+
+	gtk_label_set_text(GTK_LABEL(filer_window->toolbar_text), label);
+	g_free(label);
 }
 
+/* Create, destroy or recreate toolbar for this window so that it
+ * matches the option setting.
+ */
+void toolbar_update_toolbar(FilerWindow *filer_window)
+{
+	g_return_if_fail(filer_window != NULL);
+
+	if (filer_window->toolbar)
+	{
+		gtk_widget_destroy(filer_window->toolbar);
+		filer_window->toolbar = NULL;
+		filer_window->toolbar_text = NULL;
+	}
+
+	if (o_toolbar.int_value != TOOLBAR_NONE)
+	{
+		filer_window->toolbar = create_toolbar(filer_window);
+		gtk_box_pack_start(filer_window->toplevel_vbox,
+				filer_window->toolbar, FALSE, TRUE, 0);
+		gtk_box_reorder_child(filer_window->toplevel_vbox,
+				filer_window->toolbar, 0);
+		gtk_widget_show_all(filer_window->toolbar);
+	}
+
+	filer_target_mode(filer_window, NULL, NULL, NULL);
+	toolbar_update_info(filer_window);
+}
 
 /****************************************************************
  *			INTERNAL FUNCTIONS			*
@@ -243,10 +275,9 @@ static void toolbar_help_clicked(GtkWidget *widget, FilerWindow *filer_window)
 	event = gtk_get_current_event();
 	if (event->type == GDK_BUTTON_RELEASE &&
 			((GdkEventButton *) event)->button != 1)
-		filer_change_to(filer_window,
-				make_path(app_dir, "Help")->str, NULL);
+		menu_rox_help(NULL, HELP_MANUAL, NULL);
 	else
-		filer_opendir(make_path(app_dir, "Help")->str, NULL);
+		filer_opendir(make_path(app_dir, "Help"), NULL, NULL);
 }
 
 static void toolbar_refresh_clicked(GtkWidget *widget,
@@ -258,13 +289,10 @@ static void toolbar_refresh_clicked(GtkWidget *widget,
 	if (event->type == GDK_BUTTON_RELEASE &&
 			((GdkEventButton *) event)->button != 1)
 	{
-		filer_opendir(filer_window->path, filer_window);
+		filer_opendir(filer_window->sym_path, filer_window, NULL);
 	}
 	else
-	{
 		full_refresh();
-		filer_update_dir(filer_window, TRUE);
-	}
 }
 
 static void toolbar_home_clicked(GtkWidget *widget, FilerWindow *filer_window)
@@ -274,12 +302,32 @@ static void toolbar_home_clicked(GtkWidget *widget, FilerWindow *filer_window)
 	event = gtk_get_current_event();
 	if (event->type == GDK_BUTTON_RELEASE && NEW_WIN_BUTTON(event))
 	{
-		filer_opendir(home_dir, filer_window);
+		filer_opendir(home_dir, filer_window, NULL);
 	}
 	else
 		filer_change_to(filer_window, home_dir, NULL);
 }
 
+static void toolbar_bookmarks_clicked(GtkWidget *widget,
+				      FilerWindow *filer_window)
+{
+	GdkEvent	*event;
+
+	g_return_if_fail(filer_window != NULL);
+
+	event = gtk_get_current_event();
+	if (event->type == GDK_BUTTON_PRESS &&
+			((GdkEventButton *) event)->button == 1)
+	{
+		bookmarks_show_menu(filer_window);
+	}
+	else if (event->type == GDK_BUTTON_RELEASE &&
+			((GdkEventButton *) event)->button != 1)
+	{
+		bookmarks_edit();
+	}
+}
+
 static void toolbar_close_clicked(GtkWidget *widget, FilerWindow *filer_window)
 {
 	GdkEvent	*event;
@@ -290,9 +338,9 @@ static void toolbar_close_clicked(GtkWidget *widget, FilerWindow *filer_window)
 	if (event->type == GDK_BUTTON_RELEASE &&
 			((GdkEventButton *) event)->button != 1)
 	{
-		filer_opendir(filer_window->path, filer_window);
+		filer_opendir(filer_window->sym_path, filer_window, NULL);
 	}
-	else
+	else if (!filer_window_delete(filer_window->window, NULL, filer_window))
 		gtk_widget_destroy(filer_window->window);
 }
 
@@ -309,27 +357,36 @@ static void toolbar_up_clicked(GtkWidget *widget, FilerWindow *filer_window)
 		change_to_parent(filer_window);
 }
 
+static void toolbar_autosize_clicked(GtkWidget *widget, FilerWindow *filer_window)
+{
+	GdkEventButton	*bev;
+
+	bev = (GdkEventButton *) gtk_get_current_event();
+	if (bev->type != GDK_BUTTON_RELEASE)
+		return;
+
+	display_set_layout(filer_window, AUTO_SIZE_ICONS, filer_window->details_type,
+			   TRUE);
+}
+
 static void toolbar_size_clicked(GtkWidget *widget, FilerWindow *filer_window)
 {
 	GdkEventButton	*bev;
 
 	bev = (GdkEventButton *) gtk_get_current_event();
+	if (bev->type != GDK_BUTTON_RELEASE)
+		return;
 
-	display_change_size(filer_window, 
-		bev->type == GDK_BUTTON_RELEASE && bev->button == 1);
+	display_change_size(filer_window, bev->button == 1);
 }
 
 static void toolbar_details_clicked(GtkWidget *widget,
 				    FilerWindow *filer_window)
 {
-	if (filer_window->details_type == DETAILS_NONE)
-		display_set_layout(filer_window,
-				filer_window->display_style,
-				DETAILS_SUMMARY);
+	if (filer_window->view_type == VIEW_TYPE_DETAILS)
+		filer_set_view_type(filer_window, VIEW_TYPE_COLLECTION);
 	else
-		display_set_layout(filer_window,
-				filer_window->display_style,
-				DETAILS_NONE);
+		filer_set_view_type(filer_window, VIEW_TYPE_DETAILS);
 }
 
 static void toolbar_hidden_clicked(GtkWidget *widget,
@@ -338,50 +395,67 @@ static void toolbar_hidden_clicked(GtkWidget *widget,
 	display_set_hidden(filer_window, !filer_window->show_hidden);
 }
 
+/* If filer_window is NULL, the toolbar is for the options window */
 static GtkWidget *create_toolbar(FilerWindow *filer_window)
 {
-	GtkWidget	*box;
+	GtkWidget	*bar;
 	GtkWidget	*b;
 	int		i;
 
-	box = gtk_hbox_new(FALSE, 0);
+	bar = gtk_toolbar_new();
+	if (filer_window)
+		gtk_widget_set_size_request(bar, 100, -1);
+
+	if (o_toolbar.int_value == TOOLBAR_NORMAL || !filer_window)
+		gtk_toolbar_set_style(GTK_TOOLBAR(bar), GTK_TOOLBAR_ICONS);
+	else if (o_toolbar.int_value == TOOLBAR_HORIZONTAL)
+		gtk_toolbar_set_style(GTK_TOOLBAR(bar), GTK_TOOLBAR_BOTH_HORIZ);
+	else
+		gtk_toolbar_set_style(GTK_TOOLBAR(bar), GTK_TOOLBAR_BOTH);
 
 	for (i = 0; i < sizeof(all_tools) / sizeof(*all_tools); i++)
 	{
 		Tool	*tool = &all_tools[i];
 
-		if (!tool->enabled)
+		if (filer_window && !tool->enabled)
 			continue;
 
-		b = add_button(box, tool, filer_window);
-		if (tool->drop_action != DROP_NONE)
+		b = add_button(bar, tool, filer_window);
+		if (filer_window && tool->drop_action != DROP_NONE)
 			handle_drops(filer_window, b, tool->drop_action);
 	}
 
-	filer_window->toolbar_text = gtk_label_new("");
-	gtk_misc_set_alignment(GTK_MISC(filer_window->toolbar_text), 0, 0.5);
-	gtk_widget_set_usize(filer_window->toolbar_text, 8, 1);
-	gtk_box_pack_start(GTK_BOX(box), filer_window->toolbar_text,
-			TRUE, TRUE, 4);
+	if (filer_window)
+	{
+		filer_window->toolbar_text = gtk_label_new("");
+		gtk_misc_set_alignment(GTK_MISC(filer_window->toolbar_text),
+					0, 0.5);
+		gtk_toolbar_append_widget(GTK_TOOLBAR(bar),
+				filer_window->toolbar_text, NULL, NULL);
+	}
 
-	if (o_toolbar_info)
-		gtk_signal_connect(GTK_OBJECT(filer_window->collection),
-				"selection_changed",
-				GTK_SIGNAL_FUNC(coll_selection_changed),
-				(gpointer) filer_window);
-
-	return box;
+	return bar;
 }
 
 /* This is used to simulate a click when button 3 is used (GtkButton
  * normally ignores this).
  */
 static gint toolbar_other_button = 0;
-static gint toolbar_adjust_pressed(GtkButton *button,
+static gint toolbar_button_pressed(GtkButton *button,
 				GdkEventButton *event,
 				FilerWindow *filer_window)
 {
 	gint	b = event->button;
+	Tool	*tool;
+
+	tool = g_object_get_data(G_OBJECT(button), "rox-tool");
+	g_return_val_if_fail(tool != NULL, TRUE);
+
+	if (tool->menu && b == 1)
+	{
+		tool->clicked((GtkWidget *) button, filer_window);
+		return TRUE;
+	}
 
 	if ((b == 2 || b == 3) && toolbar_other_button == 0)
 	{
@@ -395,7 +469,7 @@ static gint toolbar_adjust_pressed(GtkButton *button,
 	return FALSE;
 }
 
-static gint toolbar_adjust_released(GtkButton *button,
+static gint toolbar_button_released(GtkButton *button,
 				GdkEventButton *event,
 				FilerWindow *filer_window)
 {
@@ -411,86 +485,67 @@ static gint toolbar_adjust_released(GtkButton *button,
 	return FALSE;
 }
 
-#if 0
-static gint menu_pressed(GtkWidget *button,
-			 GdkEventButton *event,
-			 FilerWindow *filer_window)
-{
-	GtkWidget	*menu;
-
-	if (event->button != 3 && event->button != 2)
-		return FALSE;
-
-	menu = gtk_object_get_data(GTK_OBJECT(button), "popup_menu");
-	g_return_val_if_fail(menu != NULL, TRUE);
-
-	show_style_menu(filer_window, event, menu);
-
-	return TRUE;
-}
-#endif
-
-static GtkWidget *add_button(GtkWidget *box, Tool *tool,
+/* If filer_window is NULL, the toolbar is for the options window */
+static GtkWidget *add_button(GtkWidget *bar, Tool *tool,
 				FilerWindow *filer_window)
 {
 	GtkWidget 	*button, *icon_widget;
-	GtkSignalFunc	cb = GTK_SIGNAL_FUNC(tool->clicked);
 
-	button = gtk_button_new();
-	gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
-	GTK_WIDGET_UNSET_FLAGS(button, GTK_CAN_FOCUS);
+	icon_widget = gtk_image_new_from_stock(tool->name,
+						GTK_ICON_SIZE_LARGE_TOOLBAR);
 
-	if (tool->menu)
+	button = gtk_toolbar_insert_element(GTK_TOOLBAR(bar),
+			   filer_window ? GTK_TOOLBAR_CHILD_BUTTON
+					: GTK_TOOLBAR_CHILD_TOGGLEBUTTON,
+			   NULL,
+			   _(tool->label),
+			   _(tool->tip), NULL,
+			   icon_widget,
+			   NULL, NULL,	/* CB, userdata */
+			   GTK_TOOLBAR(bar)->num_children);
+	
+	if (o_toolbar.int_value == TOOLBAR_HORIZONTAL)
 	{
-		g_warning("Not implemented");
-#if 0
-		gtk_object_set_data(GTK_OBJECT(button), "popup_menu",
-				*tool->menu);
-		gtk_signal_connect(GTK_OBJECT(button), "button_press_event",
-			GTK_SIGNAL_FUNC(menu_pressed), filer_window);
-#endif
+		GtkWidget *hbox, *label;
+		GList	  *kids;
+		hbox = GTK_BIN(button)->child;
+		kids = gtk_container_get_children(GTK_CONTAINER(hbox));
+		label = g_list_nth_data(kids, 1);
+		g_list_free(kids);
+		
+		if (label)
+		{
+			gtk_box_set_child_packing(GTK_BOX(hbox), label,
+						TRUE, TRUE, 0, GTK_PACK_END);
+			gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
+		}
+	}
+
+	g_object_set_data(G_OBJECT(button), "rox-tool", tool);
+
+	if (filer_window)
+	{
+		g_signal_connect(button, "clicked",
+			G_CALLBACK(tool->clicked), filer_window);
+		g_signal_connect(button, "button_press_event",
+			G_CALLBACK(toolbar_button_pressed), filer_window);
+		g_signal_connect(button, "button_release_event",
+			G_CALLBACK(toolbar_button_released), filer_window);
 	}
 	else
 	{
-		gtk_signal_connect(GTK_OBJECT(button), "button_press_event",
-			GTK_SIGNAL_FUNC(toolbar_adjust_pressed), filer_window);
-		gtk_signal_connect(GTK_OBJECT(button), "button_release_event",
-			GTK_SIGNAL_FUNC(toolbar_adjust_released), filer_window);
+		g_signal_connect(button, "clicked",
+			G_CALLBACK(toggle_selected), NULL);
+		g_object_set_data(G_OBJECT(button), "tool_name",
+				  (gpointer) tool->name);
 	}
 
-	gtk_signal_connect(GTK_OBJECT(button), "clicked",
-			cb, filer_window);
-
-	gtk_tooltips_set_tip(tooltips, button, _(tool->tip), NULL);
-
-	icon_widget = gtk_pixmap_new(tool->icon->pixmap, tool->icon->mask);
-
-	if (o_toolbar == TOOLBAR_LARGE)
-	{
-		GtkWidget	*vbox, *text;
-
-		vbox = gtk_vbox_new(FALSE, 0);
-		gtk_box_pack_start(GTK_BOX(vbox), icon_widget, TRUE, TRUE, 0);
-
-		text = gtk_label_new(_(tool->label));
-		gtk_box_pack_start(GTK_BOX(vbox), text, FALSE, TRUE, 0);
-
-		gtk_container_add(GTK_CONTAINER(button), vbox);
-	}
-	else
-		gtk_container_add(GTK_CONTAINER(button), icon_widget);
-
-	gtk_container_set_border_width(GTK_CONTAINER(button), 1);
-	gtk_misc_set_padding(GTK_MISC(icon_widget),
-			o_toolbar == TOOLBAR_LARGE ? 16 : 8, 1);
-	gtk_box_pack_start(GTK_BOX(box), button, FALSE, TRUE, 0);
-
 	return button;
 }
 
-static void toggle_shaded(GtkWidget *widget)
+static void toggle_selected(GtkToggleButton *widget, gpointer data)
 {
-	gtk_widget_set_sensitive(widget, !GTK_WIDGET_SENSITIVE(widget));
+	option_check_widget(&o_toolbar_disable);
 }
 
 /* Called during the drag when the mouse is in a widget registered
@@ -505,27 +560,30 @@ static gboolean drag_motion(GtkWidget		*widget,
 {
 	GdkDragAction	action = context->suggested_action;
 	DropDest	dest;
+	gpointer	type = (gpointer) drop_dest_dir;
 
-	dest = (DropDest) gtk_object_get_data(GTK_OBJECT(widget),
-							"toolbar_dest");
+	dest = (DropDest) g_object_get_data(G_OBJECT(widget), "toolbar_dest");
+
+	if ((context->actions & GDK_ACTION_ASK) && o_dnd_left_menu.int_value &&
+		dest != DROP_BOOKMARK)
+	{
+		guint state;
+		gdk_window_get_pointer(NULL, NULL, NULL, &state);
+		if (state & GDK_BUTTON1_MASK)
+			action = GDK_ACTION_ASK;
+	}
 
 	if (dest == DROP_TO_HOME)
-		g_dataset_set_data(context, "drop_dest_path", home_dir);
+		g_dataset_set_data(context, "drop_dest_path",
+				   (gchar *) home_dir);
+	else if (dest == DROP_BOOKMARK)
+		type = (gpointer) drop_dest_bookmark;
 	else
-	{
-		guchar	*slash, *path;
-
-		slash = strrchr(filer_window->path, '/');
-		if (slash == NULL || slash == filer_window->path)
-			path = g_strdup("/");
-		else
-			path = g_strndup(filer_window->path,
-					slash - filer_window->path);
 		g_dataset_set_data_full(context, "drop_dest_path",
-						path, g_free);
-	}
+				g_path_get_dirname(filer_window->sym_path),
+				g_free);
 	
-	g_dataset_set_data(context, "drop_dest_type", drop_dest_dir);
+	g_dataset_set_data(context, "drop_dest_type", type);
 	gdk_drag_status(context, action, time);
 	
 	dnd_spring_load(context, filer_window);
@@ -548,12 +606,11 @@ static void handle_drops(FilerWindow *filer_window,
 			 DropDest dest)
 {
 	make_drop_target(button, 0);
-	gtk_signal_connect(GTK_OBJECT(button), "drag_motion",
-			GTK_SIGNAL_FUNC(drag_motion), filer_window);
-	gtk_signal_connect(GTK_OBJECT(button), "drag_leave",
-			GTK_SIGNAL_FUNC(drag_leave), filer_window);
-	gtk_object_set_data(GTK_OBJECT(button), "toolbar_dest",
-			(gpointer) dest);
+	g_signal_connect(button, "drag_motion",
+			G_CALLBACK(drag_motion), filer_window);
+	g_signal_connect(button, "drag_leave",
+			G_CALLBACK(drag_leave), filer_window);
+	g_object_set_data(G_OBJECT(button), "toolbar_dest", (gpointer) dest);
 }
 
 static void tally_items(gpointer key, gpointer value, gpointer data)
@@ -565,105 +622,11 @@ static void tally_items(gpointer key, gpointer value, gpointer data)
 		(*tally)++;
 }
 
-static void coll_selection_changed(Collection *collection, guint time,
-					gpointer user_data)
-{
-	FilerWindow	*filer_window = (FilerWindow *) user_data;
-	gchar		*label;
-
-	if (filer_window->target_cb)
-		return;
-
-	if (collection->number_selected == 0)
-	{
-		gchar *s = NULL;
-
-		if (filer_window->scanning)
-		{
-			gtk_label_set_text(
-				GTK_LABEL(filer_window->toolbar_text), "");
-			return;
-		}
-
-		if (!filer_window->show_hidden)
-		{
-			GHashTable *hash = filer_window->directory->known_items;
-			int	   tally = 0;
-
-			g_hash_table_foreach(hash, tally_items, &tally);
-
-			if (tally)
-				s = g_strdup_printf(_(" (%u hidden)"), tally);
-		}
-
-		if (collection->number_of_items)
-			label = g_strdup_printf("%d %s%s",
-					collection->number_of_items,
-					collection->number_of_items != 1
-						? _("items") : _("item"),
-						s ? s : "");
-		else /* (French plurals work differently for zero) */
-			label = g_strdup_printf(_("No items%s"),
-					s ? s : "");
-		g_free(s);
-	}
-	else
-	{
-		gint	i = collection->number_of_items;
-		double	size = 0;
-
-		while (i--)
-		{
-			DirItem *item = (DirItem *) collection->items[i].data;
-			
-			if (collection->items[i].selected 
-			    && item->base_type != TYPE_DIRECTORY)
-					size += (double) item->size;
-		}
-		label = g_strdup_printf(_("%u selected (%s)"),
-				collection->number_selected,
-				format_double_size(size));
-	}
-
-	gtk_label_set_text(GTK_LABEL(filer_window->toolbar_text), label);
-	g_free(label);
-}
-
-static void recreate_toolbar(FilerWindow *filer_window)
-{
-	GtkWidget	*frame = filer_window->toolbar_frame;
-	
-	if (GTK_BIN(frame)->child)
-	{
-		filer_window->toolbar_text = NULL;
-		gtk_widget_destroy(((GtkBin *) frame)->child);
-	}
-	
-	if (o_toolbar == TOOLBAR_NONE)
-		gtk_widget_hide(frame);
-	else
-	{
-		GtkWidget	*toolbar;
-
-		toolbar = toolbar_new(filer_window);
-		gtk_container_add(GTK_CONTAINER(filer_window->toolbar_frame),
-				toolbar);
-		gtk_widget_show_all(frame);
-	}
-
-	filer_target_mode(filer_window, NULL, NULL, NULL);
-	toolbar_update_info(filer_window);
-}
-
 static void option_notify(void)
 {
-	ToolbarType	old_type = o_toolbar;
-	gint		old_info = o_toolbar_info;
-	guchar		*list;
 	int		i;
 	gboolean	changed = FALSE;
-
-	list = option_get_static_string("toolbar_disable");
+	guchar		*list = o_toolbar_disable.value;
 
 	for (i = 0; i < sizeof(all_tools) / sizeof(*all_tools); i++)
 	{
@@ -675,11 +638,8 @@ static void option_notify(void)
 		if (old != tool->enabled)
 			changed = TRUE;
 	}
-	
-	o_toolbar = option_get_int("toolbar_type");
-	o_toolbar_info = option_get_int("toolbar_show_info");
 
-	if (changed || old_type != o_toolbar || old_info != o_toolbar_info)
+	if (changed || o_toolbar.has_changed || o_toolbar_info.has_changed)
 	{
 		GList	*next;
 
@@ -687,40 +647,34 @@ static void option_notify(void)
 		{
 			FilerWindow *filer_window = (FilerWindow *) next->data;
 			
-			if (old_info && old_type != TOOLBAR_NONE)
-				gtk_signal_disconnect_by_func(
-					GTK_OBJECT(filer_window->collection),
-					GTK_SIGNAL_FUNC(coll_selection_changed),
-					(gpointer) filer_window);
-
-			recreate_toolbar(filer_window);
+			toolbar_update_toolbar(filer_window);
 		}
 	}
 }
 
-static void update_tools(OptionUI *ui, guchar *value)
+static void update_tools(Option *option)
 {
 	GList	*next, *kids;
 
-	kids = gtk_container_children(GTK_CONTAINER(ui->widget));
+	kids = gtk_container_get_children(GTK_CONTAINER(option->widget));
 
 	for (next = kids; next; next = next->next)
 	{
-		GtkWidget	*kid = (GtkWidget *) next->data;
+		GtkToggleButton	*kid = (GtkToggleButton *) next->data;
 		guchar		*name;
 
-		name = gtk_object_get_data(GTK_OBJECT(kid), "tool_name");
+		name = g_object_get_data(G_OBJECT(kid), "tool_name");
 
 		g_return_if_fail(name != NULL);
 		
-		gtk_widget_set_sensitive(GTK_BIN(kid)->child,
-					 !in_list(name, value));
+		gtk_toggle_button_set_active(kid,
+					 !in_list(name, option->value));
 	}
 
 	g_list_free(kids);
 }
 
-static guchar *read_tools(OptionUI *ui)
+static guchar *read_tools(Option *option)
 {
 	GList	*next, *kids;
 	GString	*list;
@@ -728,16 +682,16 @@ static guchar *read_tools(OptionUI *ui)
 
 	list = g_string_new(NULL);
 
-	kids = gtk_container_children(GTK_CONTAINER(ui->widget));
+	kids = gtk_container_get_children(GTK_CONTAINER(option->widget));
 
 	for (next = kids; next; next = next->next)
 	{
-		GtkObject	*kid = (GtkObject *) next->data;
+		GtkToggleButton	*kid = (GtkToggleButton *) next->data;
 		guchar		*name;
 
-		if (!GTK_WIDGET_SENSITIVE(GTK_BIN(kid)->child))
+		if (!gtk_toggle_button_get_active(kid))
 		{
-			name = gtk_object_get_data(kid, "tool_name");
+			name = g_object_get_data(G_OBJECT(kid), "tool_name");
 			g_return_val_if_fail(name != NULL, list->str);
 
 			if (list->len)
@@ -753,19 +707,17 @@ static guchar *read_tools(OptionUI *ui)
 	return retval;
 }
 
-static GList *build_tool_options(OptionUI *ui, xmlNode *node, guchar *label)
+static GList *build_tool_options(Option *option, xmlNode *node, guchar *label)
 {
-	int		i = 0;
-	GtkWidget	*hbox, *tool;
+	GtkWidget	*bar;
 
-	hbox = gtk_hbox_new(FALSE, 0);
+	g_return_val_if_fail(option != NULL, NULL);
 
-	while ((tool = toolbar_tool_option(i++)))
-		gtk_box_pack_start(GTK_BOX(hbox), tool, FALSE, TRUE, 0);
+	bar = create_toolbar(NULL);
 
-	ui->update_widget = update_tools;
-	ui->read_widget = read_tools;
-	ui->widget = hbox;
+	option->update_widget = update_tools;
+	option->read_widget = read_tools;
+	option->widget = bar;
 
-	return g_list_append(NULL, hbox);
+	return g_list_append(NULL, bar);
 }
