@@ -49,6 +49,7 @@ fi
 
 umask 022
 
+
 cat << EOF
 
 **************************************************************************
@@ -71,19 +72,33 @@ echo
 case $REPLY in
 	1) APPDIR=/usr/local/apps
 	   BINDIR=/usr/local/bin
-	   MANDIR=/usr/local/man
+	   SHAREDIR=/usr/local/share
+	   CHOICESDIR=${SHAREDIR}/Choices
+	   MANDIR=/usr/local/man	# (not under share!)
 	   ;;
 	2) APPDIR=${HOME}/Apps
 	   BINDIR=${HOME}/bin
-	   if [ ! -d ${HOME}/man ]; then
-		MANDIR=""
+	   if [ -n "$CHOICESPATH" ]; then
+		CHOICESDIR=`echo $CHOICESPATH | sed -e 's/^\([^:]*\).*$/\\1/'`
+	   fi
+	   if [ ! -n "$CHOICESDIR" ]; then
+	   	CHOICESDIR=${HOME}/Choices
+	   fi
+	   if [ -n "$XDG_DATA_HOME" ]; then
+	        SHAREDIR=${XDG_DATA_HOME}
 	   else
-		MANDIR=${HOME}/man
+	   	SHAREDIR=${HOME}/.local/share
+	   fi
+	   MANDIR=${HOME}/man
+	   if [ ! -d "$MANDIR" ]; then
+	     MANDIR=
 	   fi
 	   ;;
 	3) APPDIR=/usr/apps
 	   BINDIR=/usr/bin
-	   MANDIR=/usr/man
+	   SHAREDIR=/usr/share
+	   CHOICESDIR=${SHAREDIR}/Choices
+	   MANDIR=${SHAREDIR}/man
 	   ;;
 	4) echo "Where should the ROX-Filer application go?"
 	   get_dir "/usr/local/apps"
@@ -93,22 +108,48 @@ case $REPLY in
 	   get_dir "/usr/local/bin"
 	   BINDIR="$DIR"
 	   echo
-	   echo "Where should the manual page go?"
+	   echo "Where should the shared resources (eg, MIME data) go?"
+	   get_dir "/usr/local/share"
+	   SHAREDIR="$DIR"
+	   echo
+	   echo "Where should the default icons and run actions go?"
+	   get_dir "$SHAREDIR/Choices"
+	   CHOICESDIR="$DIR"
+	   echo
+	   echo "Where should the man pages go?"
 	   get_dir "/usr/local/man"
 	   MANDIR="$DIR"
 	   ;;
 	*) die "Invalid choice!";;
 esac
 
-cat << EOF
+MIMEDIR=${SHAREDIR}/mime
 
+MIMEINFO="${MIMEDIR}/packages/rox.xml"
+
+cat << EOF
 The application directory will be:
 	$APPDIR/ROX-Filer
 
 The launcher script will be:
 	$BINDIR/rox
 
+Icons and run actions will be in:
+	$CHOICESDIR
+
+MIME rules will be:
+	$MIMEINFO
+
 EOF
+if [ ! -f rox.1 ]; then
+	echo '*** The ROX-Filer manpage (rox.1) is missing.'
+	echo '*** It can be created from Manual.xml by running'
+	echo "*** 'make' in the ROX-Filer/src/Docs directory."
+	echo '*** The daily CVS snapshots have it ready-built'
+	echo
+	MANDIR=""
+fi
+
 if [ -n "$MANDIR" ]; then
 	echo "The manual pages will be:"
 	echo "	$MANDIR/man1/rox.1"
@@ -122,6 +163,8 @@ echo "OK?"
 confirm_or_die
 echo
 
+endir "$SHAREDIR"
+
 if [ -n "$MANDIR" ]; then
 	echo "Installing manpage..."
 	endir "$MANDIR"
@@ -131,10 +174,34 @@ if [ -n "$MANDIR" ]; then
 	ln -s "$MANDIR/man1/rox.1" "$MANDIR/man1/ROX-Filer.1" || die "Can't install manpage!"
 fi
 
+echo "Installing icons (existing icons will not be replaced)..."
+endir "$CHOICESDIR/MIME-icons"
+endir "$CHOICESDIR/MIME-types"
+cd Choices || die "Choices missing"
+for file in MIME-*/*; do
+  if [ -f "$file" ]; then
+    dest="$CHOICESDIR/$file"
+    if [ ! -f "$dest" ]; then
+      if [ ! -d "$dest" ]; then
+        echo Install $file as $dest
+        cp "$file" "$dest"
+      fi
+    fi
+  fi
+done
+cd ..
+
+endir "$MIMEDIR"
+endir "$MIMEDIR/packages"
+cp rox.xml "$MIMEINFO" || die "Failed to create $MIMEINFO"
+update-mime-database "$MIMEDIR" || die "update-mime-database failed
+Make sure you have installed the shared MIME database from:
+http://www.freedesktop.org/standards/shared-mime-info.html"
+
 echo "Installing application..."
 endir "$APPDIR"
 
-(cd ROX-Filer/src; make clean) > /dev/null
+(cd ROX-Filer/src && make clean) > /dev/null 2>&1
 if [ -d "$APPDIR/ROX-Filer" ]; then
 	echo "ROX-Filer is already installed - delete the existing"
 	echo "copy?"
@@ -155,69 +222,18 @@ EOF
 chmod a+x "$BINDIR/rox"
 
 cat << EOF
+
 Script installed. You can run the filer by simply typing 'rox'
 Make sure that $BINDIR is in your PATH though - if it isn't then
 you must use
 	\$ $BINDIR/rox
 to run it instead.
-EOF
-
-if [ ! -n "$CHOICESPATH" ]; then
-	CHOICESPATH=${HOME}/Choices:/usr/local/share/Choices:/usr/share/Choices
-fi
-
-cat << EOF
-
-
-ROX-Filer requires some icons and other defaults. These should be
-installed where it can find them. By default, it looks in
-	\${HOME}/Choices (${HOME}/Choices in your case)
-	/usr/local/share/Choices
-	/usr/share/Choices
-in that order. You can choose a different search path by setting
-the CHOICESPATH environment variable before running the filer. These
-files are supplied in the 'rox-base' package. Make sure you have the
-latest version installed!
-
-EOF
-
-IFS=":"
-REPORT="I couldn't find the required files!"
-
-for DIR in $CHOICESPATH; do
-  if [ ! -n "$DIR" ]; then
-    continue
-  fi
-  
-  echo Looking for files in $DIR...
-
-  if [ -f "$DIR/MIME-icons/special_executable.xpm" ]; then
-    REPORT=""
-    echo "Found them!"
-    break
-  elif [ -d "$DIR/MIME-icons" ]; then
-    REPORT="I found an old installation in $DIR, but not the latest version."
-  fi
-done
-
-if [ -n "$REPORT" ]; then
-  echo
-  echo $REPORT
-  echo "Please download and install the latest version of the rox-base package"
-  echo "from http://rox.sourceforge.net before using the filer!"
-  echo
-  echo "Continue?"
-  confirm_or_die
-fi
-
-cat << EOF
-
 
 	****************************
 	*** Now read the manual! ***
 	****************************
 
-Run ROX and click on the information icon on the toolbar:
+Run ROX and click on the help icon on the toolbar:
 
 	\$ rox
 EOF
